// development version of telecine controller
// for 16F876 PIC Controller

#include <htc.h>
#include "uart.h"
#include "pwm.h"
#include "discrete.h"
#include "adc.h"

// flags for debug printing
#define adcbug 0	// pot positions
#define tmrbug 0	// timer (motor position)

void status(void);
void reset(void);
void stp_film(void);
unsigned char cycle_L, cycle_R, tenflg, rewflg;

void main(void){

unsigned char input, j, plft, prgt, adcch, adcval;
char fin_str[] = "fin";

	__CONFIG( XT & WDTDIS & PWRTEN & LVPDIS );
	INTCON = 0;	// disable the interrupts.
	ADCON1 = 7;	// switch all pins in port A to digital (otherwise it tries to do A/D)

// do initializations
	discrete_init();
	uart_init();
	pwm_init();
	stp_film();	//stop film
	adc_init();
	adcch = 0;
	adc_cvt(0);

// output initial reset message
	reset();
// Loop forever
	while(1){

// test for serial port traffic
		if ( RCIF == 1 ) {	//got a character
			input = getch();	// read character
// advance film
			if (input == 'a' ) {
				T1CON = 0x00;	// disable tmr0
				TMR1H = 130;	// preset count (approx 7.2 in of super 8)
				TMR1L = 0;
				TMR1IF = 0;
				T1CON = 0x07;	// enable tmr0 as counter
				RB4 = 0;	//turn on motor
				}
// calibration for servo position
			else if (input == 'c' ) {
				pwm_dc(0,0x30);
				pwm_dc(1,0x30);
				while ( !RCIF ) continue;	//wait here for a char
				}
			else {	putch (input);
				putch(0x0d); 	//cr
				putch(0x0a);	//lf
				}
			}
// test for button push
		if ( swrd(0) == 0 ) { 
			j = 100;
			while (j>0) j--;
			while ( !swrd(0) ) continue;
			blink (0);
			tenflg = 1;	// normal tension
			rewflg = 0;
			}
		if ( swrd(1) == 0 ) { 
			j = 100;
			while (j>0) j--;
			while ( !swrd(1) ) continue;
			blink (1);
			tenflg = 0;	// rewind
			rewflg = 1;
			}
// check for timer1 overflow - completion of film advance
		if ( TMR1IF ) {
			RB4 = 1;	//turn off motor
			TMR1IF = 0;
			T1CON = 0x00;
			if ( tmrbug ) {
				putch ('X');
				putch (' ');
				putch ('0' + TMR1IF );		put3digits (TMR1H);
				putch (' ');
				put3digits (TMR1H);
				putch (' ');
				put3digits (TMR1L);
				putch(0x0d); 	//cr
				putch(0x0a);}	//lf
			puts (fin_str);
			putch(0x0d); //cr
			putch(0x0a); //lf
			}
// do a/d converstions for pot position
		if ( (ADCON0 & 0x4) == 0 ){
			adcval = adc_val();
			if (adcch == 0 ){
				if (adcbug) { 
					if (adcval != prgt) { 
					prgt = adcval;
					putch('R');
					put3digits (adcval);
					putch(0x0d); 	//cr
					putch(0x0a); }	//lf
					}
				if (tenflg){
					if (adcval > 150 ) cycle_R = 0x29;
					else cycle_R = 0x30;
					}
				if (rewflg){
					if (adcval < 140 ) cycle_R = 0x33;
					else cycle_R = 0x30;
					}
				pwm_dc(1,cycle_R);
				adcch = 1; }
			else {
				if (adcbug) {
					if ( adcval != plft ) {
					plft = adcval;
					putch('L');
					put3digits (adcval);
					putch(0x0d); 	//cr
					putch(0x0a); }	//lf
					}
				if (tenflg){
					if (adcval > 160 ) cycle_L = 0x29;
					else cycle_L = 0x30;
					}
				if (rewflg){
					cycle_L = 0x32;
					}
				pwm_dc(0,cycle_L);
				adcch = 0; }
			adc_cvt(adcch);
			}

	} 
}

void stp_film(void){
	cycle_R = 0x30;	//initial duty cycle
	cycle_L = 0x30;
	pwm_dc(0,0x30);
	pwm_dc(1,0x30);
	tenflg = 0;	// tension tape flag (normal tape ops)
	rewflg = 0;	// rewind tape flag
	}
	
void reset (void) {
char reset_str[] = "reset";
	puts (reset_str);
	putch(0x0d); //cr
	putch(0x0a); //lf
}

void status (void) {
char status_str[] = "status";
	puts (status_str);
	putch(0x0d); //cr
	putch(0x0a); //lf
}



